@@ -15,7 +15,7 @@ Example:
 
   >>> import smtplib
   >>> s=smtplib.SMTP("localhost")
-  >>> print s.help()
+  >>> print(s.help())
   This is Sendmail version 8.8.4
   Topics:
       HELO    EHLO    MAIL    RCPT    DATA
@@ -46,22 +46,20 @@ import re
 import email.utils
 import base64
 import hmac
-from email.base64mime import encode as encode_base64
+from email.base64mime import body_encode as encode_base64
 from sys import stderr
 
-__all__ = ["SMTPException", "SMTPServerDisconnected", "SMTPResponseException",
-           "SMTPSenderRefused", "SMTPRecipientsRefused", "SMTPDataError",
-           "SMTPConnectError", "SMTPHeloError", "SMTPAuthenticationError",
-           "quoteaddr", "quotedata", "SMTP"]
+__all__ = ["SMTPException","SMTPServerDisconnected","SMTPResponseException",
+           "SMTPSenderRefused","SMTPRecipientsRefused","SMTPDataError",
+           "SMTPConnectError","SMTPHeloError","SMTPAuthenticationError",
+           "quoteaddr","quotedata","SMTP"]
 
 SMTP_PORT = 25
 SMTP_SSL_PORT = 465
-CRLF = "\r\n"
-_MAXLINE = 8192 # more than 8 times larger than RFC 821, 4.5.3
+CRLF="\r\n"
 
 OLDSTYLE_AUTH = re.compile(r"auth=(.*)", re.I)
 
-
 # Exception classes used by this module.
 class SMTPException(Exception):
     """Base class for all exceptions raised by this module."""
@@ -111,7 +109,7 @@ class SMTPRecipientsRefused(SMTPException):
 
     def __init__(self, recipients):
         self.recipients = recipients
-        self.args = (recipients,)
+        self.args = ( recipients,)
 
 
 class SMTPDataError(SMTPResponseException):
@@ -130,18 +128,17 @@ class SMTPAuthenticationError(SMTPResponseException):
     combination provided.
     """
 
-
 def quoteaddr(addr):
     """Quote a subset of the email addresses defined by RFC 821.
 
-    Should be able to handle anything rfc822.parseaddr can handle.
+    Should be able to handle anything email.utils.parseaddr can handle.
     """
     m = (None, None)
     try:
         m = email.utils.parseaddr(addr)[1]
     except AttributeError:
         pass
-    if m == (None, None):  # Indicates parse failure or AttributeError
+    if m == (None, None): # Indicates parse failure or AttributeError
         # something weird here.. punt -ddm
         return "<%s>" % addr
     elif m is None:
@@ -150,13 +147,6 @@ def quoteaddr(addr):
     else:
         return "<%s>" % m
 
-def _addr_only(addrstring):
-    displayname, addr = email.utils.parseaddr(addrstring)
-    if (displayname, addr) == ('', ''):
-        # parseaddr couldn't parse it, so use it as is.
-        return addrstring
-    return addr
-
 def quotedata(data):
     """Quote data for email.
 
@@ -166,7 +156,6 @@ def quotedata(data):
     return re.sub(r'(?m)^\.', '..',
         re.sub(r'(?:\r\n|\n|\r(?!\n))', CRLF, data))
 
-
 try:
     import ssl
 except ImportError:
@@ -180,17 +169,12 @@ else:
         def __init__(self, sslobj):
             self.sslobj = sslobj
 
-        def readline(self, size=-1):
-            if size < 0:
-                size = None
-            str = ""
+        def readline(self):
+            str = b""
             chr = None
-            while chr != "\n":
-                if size is not None and len(str) >= size:
-                    break
+            while chr != b"\n":
                 chr = self.sslobj.read(1)
-                if not chr:
-                    break
+                if not chr: break
                 str += chr
             return str
 
@@ -199,6 +183,7 @@ else:
 
     _have_ssl = True
 
+
 class SMTP:
     """This class manages a connection to an SMTP or ESMTP server.
     SMTP Objects:
@@ -242,12 +227,10 @@ class SMTP:
 
         If specified, `host' is the name of the remote host to which to
         connect.  If specified, `port' specifies the port to which to connect.
-        By default, smtplib.SMTP_PORT is used.  If a host is specified the
-        connect method is called, and if it returns anything other than a
-        success code an SMTPConnectError is raised.  If specified,
-        `local_hostname` is used as the FQDN of the local host for the
-        HELO/EHLO command.  Otherwise, the local hostname is found using
-        socket.getfqdn().
+        By default, smtplib.SMTP_PORT is used.  An SMTPConnectError is raised
+        if the specified `host' doesn't respond correctly.  If specified,
+        `local_hostname` is used as the FQDN of the local host.  By default,
+        the local hostname is found using socket.getfqdn().
 
         """
         self.timeout = timeout
@@ -286,11 +269,10 @@ class SMTP:
     def _get_socket(self, host, port, timeout):
         # This makes it simpler for SMTP_SSL to use the SMTP connect code
         # and just alter the socket connection bit.
-        if self.debuglevel > 0:
-            print>>stderr, 'connect:', (host, port)
+        if self.debuglevel > 0: print('connect:', (host, port), file=stderr)
         return socket.create_connection((host, port), timeout)
 
-    def connect(self, host='localhost', port=0):
+    def connect(self, host='localhost', port = 0):
         """Connect to a host on a given port.
 
         If the hostname ends with a colon (`:') followed by a number, and
@@ -304,28 +286,25 @@ class SMTP:
         if not port and (host.find(':') == host.rfind(':')):
             i = host.rfind(':')
             if i >= 0:
-                host, port = host[:i], host[i + 1:]
-                try:
-                    port = int(port)
+                host, port = host[:i], host[i+1:]
+                try: port = int(port)
                 except ValueError:
-                    raise socket.error, "nonnumeric port"
-        if not port:
-            port = self.default_port
-        if self.debuglevel > 0:
-            print>>stderr, 'connect:', (host, port)
+                    raise socket.error("nonnumeric port")
+        if not port: port = self.default_port
+        if self.debuglevel > 0: print('connect:', (host, port), file=stderr)
         self.sock = self._get_socket(host, port, self.timeout)
         (code, msg) = self.getreply()
-        if self.debuglevel > 0:
-            print>>stderr, "connect:", msg
+        if self.debuglevel > 0: print("connect:", msg, file=stderr)
         return (code, msg)
 
-    def send(self, str):
-        """Send `str' to the server."""
-        if self.debuglevel > 0:
-            print>>stderr, 'send:', repr(str)
+    def send(self, s):
+        """Send `s' to the server."""
+        if self.debuglevel > 0: print('send:', repr(s), file=stderr)
         if hasattr(self, 'sock') and self.sock:
+            if isinstance(s, str):
+                s = s.encode("ascii")
             try:
-                self.sock.sendall(str)
+                self.sock.sendall(s)
             except socket.error:
                 self.close()
                 raise SMTPServerDisconnected('Server not connected')
@@ -353,25 +332,20 @@ class SMTP:
 
         Raises SMTPServerDisconnected if end-of-file is reached.
         """
-        resp = []
+        resp=[]
         if self.file is None:
             self.file = self.sock.makefile('rb')
         while 1:
             try:
-                line = self.file.readline(_MAXLINE + 1)
-            except socket.error as e:
-                self.close()
-                raise SMTPServerDisconnected("Connection unexpectedly closed: "
-                                             + str(e))
-            if line == '':
+                line = self.file.readline()
+            except socket.error:
+                line = ''
+            if not line:
                 self.close()
                 raise SMTPServerDisconnected("Connection unexpectedly closed")
-            if self.debuglevel > 0:
-                print>>stderr, 'reply:', repr(line)
-            if len(line) > _MAXLINE:
-                raise SMTPResponseException(500, "Line too long.")
-            resp.append(line[4:].strip())
-            code = line[:3]
+            if self.debuglevel > 0: print('reply:', repr(line), file=stderr)
+            resp.append(line[4:].strip(b' \t\r\n'))
+            code=line[:3]
             # Check that the error code is syntactically correct.
             # Don't attempt to read a continuation line if it is broken.
             try:
@@ -380,17 +354,17 @@ class SMTP:
                 errcode = -1
                 break
             # Check if multiline response.
-            if line[3:4] != "-":
+            if line[3:4] != b"-":
                 break
 
-        errmsg = "\n".join(resp)
+        errmsg = b"\n".join(resp)
         if self.debuglevel > 0:
-            print>>stderr, 'reply: retcode (%s); Msg: %s' % (errcode, errmsg)
+            print('reply: retcode (%s); Msg: %s' % (errcode,errmsg), file=stderr)
         return errcode, errmsg
 
     def docmd(self, cmd, args=""):
         """Send a command, and return its response code."""
-        self.putcmd(cmd, args)
+        self.putcmd(cmd,args)
         return self.getreply()
 
     # std smtp commands
@@ -400,9 +374,9 @@ class SMTP:
         host.
         """
         self.putcmd("helo", name or self.local_hostname)
-        (code, msg) = self.getreply()
-        self.helo_resp = msg
-        return (code, msg)
+        (code,msg)=self.getreply()
+        self.helo_resp=msg
+        return (code,msg)
 
     def ehlo(self, name=''):
         """ SMTP 'ehlo' command.
@@ -411,19 +385,20 @@ class SMTP:
         """
         self.esmtp_features = {}
         self.putcmd(self.ehlo_msg, name or self.local_hostname)
-        (code, msg) = self.getreply()
+        (code,msg)=self.getreply()
         # According to RFC1869 some (badly written)
         # MTA's will disconnect on an ehlo. Toss an exception if
         # that happens -ddm
         if code == -1 and len(msg) == 0:
             self.close()
             raise SMTPServerDisconnected("Server not connected")
-        self.ehlo_resp = msg
+        self.ehlo_resp=msg
         if code != 250:
-            return (code, msg)
-        self.does_esmtp = 1
+            return (code,msg)
+        self.does_esmtp=1
         #parse the ehlo response -ddm
-        resp = self.ehlo_resp.split('\n')
+        assert isinstance(self.ehlo_resp, bytes), repr(self.ehlo_resp)
+        resp=self.ehlo_resp.decode("latin-1").split('\n')
         del resp[0]
         for each in resp:
             # To be able to communicate with as many SMTP servers as possible,
@@ -443,16 +418,16 @@ class SMTP:
             # It's actually stricter, in that only spaces are allowed between
             # parameters, but were not going to check for that here.  Note
             # that the space isn't present if there are no parameters.
-            m = re.match(r'(?P<feature>[A-Za-z0-9][A-Za-z0-9\-]*) ?', each)
+            m=re.match(r'(?P<feature>[A-Za-z0-9][A-Za-z0-9\-]*) ?',each)
             if m:
-                feature = m.group("feature").lower()
-                params = m.string[m.end("feature"):].strip()
+                feature=m.group("feature").lower()
+                params=m.string[m.end("feature"):].strip()
                 if feature == "auth":
                     self.esmtp_features[feature] = self.esmtp_features.get(feature, "") \
                             + " " + params
                 else:
-                    self.esmtp_features[feature] = params
-        return (code, msg)
+                    self.esmtp_features[feature]=params
+        return (code,msg)
 
     def has_extn(self, opt):
         """Does the server support a given SMTP service extension?"""
@@ -472,23 +447,23 @@ class SMTP:
         """SMTP 'noop' command -- doesn't do anything :>"""
         return self.docmd("noop")
 
-    def mail(self, sender, options=[]):
+    def mail(self,sender,options=[]):
         """SMTP 'mail' command -- begins mail xfer session."""
         optionlist = ''
         if options and self.does_esmtp:
             optionlist = ' ' + ' '.join(options)
-        self.putcmd("mail", "FROM:%s%s" % (quoteaddr(sender), optionlist))
+        self.putcmd("mail", "FROM:%s%s" % (quoteaddr(sender) ,optionlist))
         return self.getreply()
 
-    def rcpt(self, recip, options=[]):
+    def rcpt(self,recip,options=[]):
         """SMTP 'rcpt' command -- indicates 1 recipient for this mail."""
         optionlist = ''
         if options and self.does_esmtp:
             optionlist = ' ' + ' '.join(options)
-        self.putcmd("rcpt", "TO:%s%s" % (quoteaddr(recip), optionlist))
+        self.putcmd("rcpt","TO:%s%s" % (quoteaddr(recip),optionlist))
         return self.getreply()
 
-    def data(self, msg):
+    def data(self,msg):
         """SMTP 'DATA' command -- sends message data to server.
 
         Automatically quotes lines beginning with a period per rfc821.
@@ -497,32 +472,30 @@ class SMTP:
         response code received when the all data is sent.
         """
         self.putcmd("data")
-        (code, repl) = self.getreply()
-        if self.debuglevel > 0:
-            print>>stderr, "data:", (code, repl)
+        (code,repl)=self.getreply()
+        if self.debuglevel >0 : print("data:", (code,repl), file=stderr)
         if code != 354:
-            raise SMTPDataError(code, repl)
+            raise SMTPDataError(code,repl)
         else:
             q = quotedata(msg)
             if q[-2:] != CRLF:
                 q = q + CRLF
             q = q + "." + CRLF
             self.send(q)
-            (code, msg) = self.getreply()
-            if self.debuglevel > 0:
-                print>>stderr, "data:", (code, msg)
-            return (code, msg)
+            (code,msg)=self.getreply()
+            if self.debuglevel >0 : print("data:", (code,msg), file=stderr)
+            return (code,msg)
 
     def verify(self, address):
         """SMTP 'verify' command -- checks for address validity."""
-        self.putcmd("vrfy", _addr_only(address))
+        self.putcmd("vrfy", quoteaddr(address))
         return self.getreply()
     # a.k.a.
-    vrfy = verify
+    vrfy=verify
 
     def expn(self, address):
         """SMTP 'expn' command -- expands a mailing list."""
-        self.putcmd("expn", _addr_only(address))
+        self.putcmd("expn", quoteaddr(address))
         return self.getreply()
 
     # some useful methods
@@ -567,12 +540,14 @@ class SMTP:
         """
 
         def encode_cram_md5(challenge, user, password):
-            challenge = base64.decodestring(challenge)
-            response = user + " " + hmac.HMAC(password, challenge).hexdigest()
-            return encode_base64(response, eol="")
+            challenge = base64.decodebytes(challenge)
+            response = user + " " + hmac.HMAC(password.encode('ascii'),
+                                              challenge).hexdigest()
+            return encode_base64(response.encode('ascii'), eol='')
 
         def encode_plain(user, password):
-            return encode_base64("\0%s\0%s" % (user, password), eol="")
+            s = "\0%s\0%s" % (user, password)
+            return encode_base64(s.encode('ascii'), eol='')
 
 
         AUTH_PLAIN = "PLAIN"
@@ -610,10 +585,10 @@ class SMTP:
                 AUTH_PLAIN + " " + encode_plain(user, password))
         elif authmethod == AUTH_LOGIN:
             (code, resp) = self.docmd("AUTH",
-                "%s %s" % (AUTH_LOGIN, encode_base64(user, eol="")))
+                "%s %s" % (AUTH_LOGIN, encode_base64(user.encode('ascii'), eol='')))
             if code != 334:
                 raise SMTPAuthenticationError(code, resp)
-            (code, resp) = self.docmd(encode_base64(password, eol=""))
+            (code, resp) = self.docmd(encode_base64(password.encode('ascii'), eol=''))
         elif authmethod is None:
             raise SMTPException("No suitable authentication method found.")
         if code not in (235, 503):
@@ -622,7 +597,7 @@ class SMTP:
             raise SMTPAuthenticationError(code, resp)
         return (code, resp)
 
-    def starttls(self, keyfile=None, certfile=None):
+    def starttls(self, keyfile = None, certfile = None):
         """Puts the connection to the SMTP server into TLS mode.
 
         If there has been no previous EHLO or HELO command this session, this
@@ -725,22 +700,22 @@ class SMTP:
             for option in mail_options:
                 esmtp_opts.append(option)
 
-        (code, resp) = self.mail(from_addr, esmtp_opts)
+        (code,resp) = self.mail(from_addr, esmtp_opts)
         if code != 250:
             self.rset()
             raise SMTPSenderRefused(code, resp, from_addr)
-        senderrs = {}
-        if isinstance(to_addrs, basestring):
+        senderrs={}
+        if isinstance(to_addrs, str):
             to_addrs = [to_addrs]
         for each in to_addrs:
-            (code, resp) = self.rcpt(each, rcpt_options)
+            (code,resp)=self.rcpt(each, rcpt_options)
             if (code != 250) and (code != 251):
-                senderrs[each] = (code, resp)
-        if len(senderrs) == len(to_addrs):
+                senderrs[each]=(code,resp)
+        if len(senderrs)==len(to_addrs):
             # the server refused all our recipients
             self.rset()
             raise SMTPRecipientsRefused(senderrs)
-        (code, resp) = self.data(msg)
+        (code,resp) = self.data(msg)
         if code != 250:
             self.rset()
             raise SMTPDataError(code, resp)
@@ -750,40 +725,29 @@ class SMTP:
 
     def close(self):
         """Close the connection to the SMTP server."""
-        try:
-            file = self.file
-            self.file = None
-            if file:
-                file.close()
-        finally:
-            sock = self.sock
-            self.sock = None
-            if sock:
-                sock.close()
+        if self.file:
+            self.file.close()
+        self.file = None
+        if self.sock:
+            self.sock.close()
+        self.sock = None
 
 
     def quit(self):
         """Terminate the SMTP session."""
         res = self.docmd("quit")
-        # A new EHLO is required after reconnecting with connect()
-        self.ehlo_resp = self.helo_resp = None
-        self.esmtp_features = {}
-        self.does_esmtp = False
         self.close()
         return res
 
 if _have_ssl:
 
     class SMTP_SSL(SMTP):
-        """ This is a subclass derived from SMTP that connects over an SSL
-        encrypted socket (to use this class you need a socket module that was
-        compiled with SSL support). If host is not specified, '' (the local
-        host) is used. If port is omitted, the standard SMTP-over-SSL port
-        (465) is used.  local_hostname has the same meaning as it does in the
-        SMTP class.  keyfile and certfile are also optional - they can contain
-        a PEM formatted private key and certificate chain file for the SSL
-        connection.
-
+        """ This is a subclass derived from SMTP that connects over an SSL encrypted
+        socket (to use this class you need a socket module that was compiled with SSL
+        support). If host is not specified, '' (the local host) is used. If port is
+        omitted, the standard SMTP-over-SSL port (465) is used. keyfile and certfile
+        are also optional - they can contain a PEM formatted private key and
+        certificate chain file for the SSL connection.
         """
 
         default_port = SMTP_SSL_PORT
@@ -796,8 +760,7 @@ if _have_ssl:
             SMTP.__init__(self, host, port, local_hostname, timeout)
 
         def _get_socket(self, host, port, timeout):
-            if self.debuglevel > 0:
-                print>>stderr, 'connect:', (host, port)
+            if self.debuglevel > 0: print('connect:', (host, port), file=stderr)
             new_socket = socket.create_connection((host, port), timeout)
             new_socket = ssl.wrap_socket(new_socket, self.keyfile, self.certfile)
             self.file = SSLFakeFile(new_socket)
@@ -814,10 +777,9 @@ class LMTP(SMTP):
     """LMTP - Local Mail Transfer Protocol
 
     The LMTP protocol, which is very similar to ESMTP, is heavily based
-    on the standard SMTP client. It's common to use Unix sockets for
-    LMTP, so our connect() method must support that as well as a regular
-    host:port server.  local_hostname has the same meaning as it does in
-    the SMTP class.  To specify a Unix socket, you must use an absolute
+    on the standard SMTP client. It's common to use Unix sockets for LMTP,
+    so our connect() method must support that as well as a regular
+    host:port server. To specify a Unix socket, you must use an absolute
     path as the host, starting with a '/'.
 
     Authentication is supported, using the regular SMTP mechanism. When
@@ -826,11 +788,11 @@ class LMTP(SMTP):
 
     ehlo_msg = "lhlo"
 
-    def __init__(self, host='', port=LMTP_PORT, local_hostname=None):
+    def __init__(self, host = '', port = LMTP_PORT, local_hostname = None):
         """Initialize a new instance."""
         SMTP.__init__(self, host, port, local_hostname)
 
-    def connect(self, host='localhost', port=0):
+    def connect(self, host = 'localhost', port = 0):
         """Connect to the LMTP daemon, on either a Unix or a TCP socket."""
         if host[0] != '/':
             return SMTP.connect(self, host, port)
@@ -839,16 +801,14 @@ class LMTP(SMTP):
         try:
             self.sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
             self.sock.connect(host)
-        except socket.error:
-            if self.debuglevel > 0:
-                print>>stderr, 'connect fail:', host
+        except socket.error as msg:
+            if self.debuglevel > 0: print('connect fail:', host, file=stderr)
             if self.sock:
                 self.sock.close()
             self.sock = None
-            raise
+            raise socket.error(msg)
         (code, msg) = self.getreply()
-        if self.debuglevel > 0:
-            print>>stderr, "connect:", msg
+        if self.debuglevel > 0: print('connect:', msg, file=stderr)
         return (code, msg)
 
 
@@ -862,15 +822,15 @@ if __name__ == '__main__':
         return sys.stdin.readline().strip()
 
     fromaddr = prompt("From")
-    toaddrs = prompt("To").split(',')
-    print "Enter message, end with ^D:"
+    toaddrs  = prompt("To").split(',')
+    print("Enter message, end with ^D:")
     msg = ''
     while 1:
         line = sys.stdin.readline()
         if not line:
             break
         msg = msg + line
-    print "Message length is %d" % len(msg)
+    print("Message length is %d" % len(msg))
 
     server = SMTP('localhost')
     server.set_debuglevel(1)
