@@ -1,5 +1,5 @@
-from test.test_support import run_unittest
-from _locale import (setlocale, LC_NUMERIC, localeconv, Error)
+from test.support import verbose, run_unittest
+from _locale import (setlocale, LC_ALL, LC_CTYPE, LC_NUMERIC, localeconv, Error)
 try:
     from _locale import (RADIXCHAR, THOUSEP, nl_langinfo)
 except ImportError:
@@ -22,9 +22,8 @@ candidate_locales = ['es_UY', 'fr_FR', 'fi_FI', 'es_CO', 'pt_PT', 'it_IT',
     'da_DK', 'nn_NO', 'cs_CZ', 'de_LU', 'es_BO', 'sq_AL', 'sk_SK', 'fr_CH',
     'de_DE', 'sr_YU', 'br_FR', 'nl_BE', 'sv_FI', 'pl_PL', 'fr_CA', 'fo_FO',
     'bs_BA', 'fr_LU', 'kl_GL', 'fa_IR', 'de_BE', 'sv_SE', 'it_CH', 'uk_UA',
-    'eu_ES', 'vi_VN', 'af_ZA', 'nb_NO', 'en_DK', 'tg_TJ', 'ps_AF.UTF-8', 'en_US',
-    'fr_FR.ISO8859-1', 'fr_FR.UTF-8', 'fr_FR.ISO8859-15@euro',
-    'ru_RU.KOI8-R', 'ko_KR.eucKR']
+    'eu_ES', 'vi_VN', 'af_ZA', 'nb_NO', 'en_DK', 'tg_TJ', 'en_US',
+    'es_ES.ISO8859-1', 'fr_FR.ISO8859-15', 'ru_RU.KOI8-R', 'ko_KR.eucKR']
 
 # Workaround for MSVC6(debug) crash bug
 if "MSC v.1200" in sys.version:
@@ -36,20 +35,15 @@ if "MSC v.1200" in sys.version:
 # List known locale values to test against when available.
 # Dict formatted as ``<locale> : (<decimal_point>, <thousands_sep>)``.  If a
 # value is not known, use '' .
-known_numerics = {
-    'en_US': ('.', ','),
-    'de_DE' : (',', '.'),
-    'fr_FR.UTF-8' : (',', ' '),
-    'ps_AF.UTF-8' : ('\xd9\xab', '\xd9\xac'),
-}
+known_numerics = {'fr_FR' : (',', ''), 'en_US':('.', ',')}
 
 class _LocaleTests(unittest.TestCase):
 
     def setUp(self):
-        self.oldlocale = setlocale(LC_NUMERIC)
+        self.oldlocale = setlocale(LC_ALL)
 
     def tearDown(self):
-        setlocale(LC_NUMERIC, self.oldlocale)
+        setlocale(LC_ALL, self.oldlocale)
 
     # Want to know what value was calculated, what it was compared against,
     # what function was used for the calculation, what type of data was used,
@@ -70,46 +64,39 @@ class _LocaleTests(unittest.TestCase):
                                     calc_value, known_value,
                                     calc_type, data_type, set_locale,
                                     used_locale))
-            return True
 
     @unittest.skipUnless(nl_langinfo, "nl_langinfo is not available")
     def test_lc_numeric_nl_langinfo(self):
         # Test nl_langinfo against known values
-        tested = False
         for loc in candidate_locales:
             try:
                 setlocale(LC_NUMERIC, loc)
+                setlocale(LC_CTYPE, loc)
             except Error:
                 continue
             for li, lc in ((RADIXCHAR, "decimal_point"),
                             (THOUSEP, "thousands_sep")):
-                if self.numeric_tester('nl_langinfo', nl_langinfo(li), lc, loc):
-                    tested = True
-        if not tested:
-            self.skipTest('no suitable locales')
+                self.numeric_tester('nl_langinfo', nl_langinfo(li), lc, loc)
 
     def test_lc_numeric_localeconv(self):
         # Test localeconv against known values
-        tested = False
         for loc in candidate_locales:
             try:
                 setlocale(LC_NUMERIC, loc)
+                setlocale(LC_CTYPE, loc)
             except Error:
                 continue
-            formatting = localeconv()
-            for lc in ("decimal_point", "thousands_sep"):
-                if self.numeric_tester('localeconv', formatting[lc], lc, loc):
-                    tested = True
-        if not tested:
-            self.skipTest('no suitable locales')
+            for lc in ("decimal_point",
+                        "thousands_sep"):
+                self.numeric_tester('localeconv', localeconv()[lc], lc, loc)
 
     @unittest.skipUnless(nl_langinfo, "nl_langinfo is not available")
     def test_lc_numeric_basic(self):
         # Test nl_langinfo against localeconv
-        tested = False
         for loc in candidate_locales:
             try:
                 setlocale(LC_NUMERIC, loc)
+                setlocale(LC_CTYPE, loc)
             except Error:
                 continue
             for li, lc in ((RADIXCHAR, "decimal_point"),
@@ -125,17 +112,14 @@ class _LocaleTests(unittest.TestCase):
                                 "(set to %s, using %s)" % (
                                                 nl_radixchar, li_radixchar,
                                                 loc, set_locale))
-                tested = True
-        if not tested:
-            self.skipTest('no suitable locales')
 
     def test_float_parsing(self):
         # Bug #1391872: Test whether float parsing is okay on European
         # locales.
-        tested = False
         for loc in candidate_locales:
             try:
                 setlocale(LC_NUMERIC, loc)
+                setlocale(LC_CTYPE, loc)
             except Error:
                 continue
 
@@ -150,10 +134,6 @@ class _LocaleTests(unittest.TestCase):
             if localeconv()['decimal_point'] != '.':
                 self.assertRaises(ValueError, float,
                                   localeconv()['decimal_point'].join(['1', '23']))
-            tested = True
-        if not tested:
-            self.skipTest('no suitable locales')
-
 
 def test_main():
     run_unittest(_LocaleTests)
