@@ -1,23 +1,17 @@
 # Very rudimentary test of threading module
 
-import test.test_support
-from test.test_support import verbose, cpython_only
-from test.script_helper import assert_python_ok
-
+import test.support
+from test.support import verbose
+import os
 import random
 import re
 import sys
-thread = test.test_support.import_module('thread')
-threading = test.test_support.import_module('threading')
+import threading
+import _thread
 import time
 import unittest
 import weakref
-import os
 import subprocess
-try:
-    import _testcapi
-except ImportError:
-    _testcapi = None
 
 from test import lock_tests
 
@@ -43,37 +37,29 @@ class TestThread(threading.Thread):
     def run(self):
         delay = random.random() / 10000.0
         if verbose:
-            print 'task %s will run for %.1f usec' % (
-                self.name, delay * 1e6)
+            print('task %s will run for %.1f usec' %
+                  (self.name, delay * 1e6))
 
         with self.sema:
             with self.mutex:
                 self.nrunning.inc()
                 if verbose:
-                    print self.nrunning.get(), 'tasks are running'
+                    print(self.nrunning.get(), 'tasks are running')
                 self.testcase.assertTrue(self.nrunning.get() <= 3)
 
             time.sleep(delay)
             if verbose:
-                print 'task', self.name, 'done'
+                print('task', self.name, 'done')
 
             with self.mutex:
                 self.nrunning.dec()
                 self.testcase.assertTrue(self.nrunning.get() >= 0)
                 if verbose:
-                    print '%s is finished. %d tasks are running' % (
-                        self.name, self.nrunning.get())
-
-class BaseTestCase(unittest.TestCase):
-    def setUp(self):
-        self._threads = test.test_support.threading_setup()
-
-    def tearDown(self):
-        test.test_support.threading_cleanup(*self._threads)
-        test.test_support.reap_children()
+                    print('%s is finished. %d tasks are running' %
+                          (self.name, self.nrunning.get()))
 
 
-class ThreadTests(BaseTestCase):
+class ThreadTests(unittest.TestCase):
 
     # Create a bunch of threads, let each do some work, wait until all are
     # done.
@@ -97,7 +83,7 @@ class ThreadTests(BaseTestCase):
             t.start()
 
         if verbose:
-            print 'waiting for all tasks to complete'
+            print('waiting for all tasks to complete')
         for t in threads:
             t.join(NUMTASKS)
             self.assertTrue(not t.is_alive())
@@ -105,7 +91,7 @@ class ThreadTests(BaseTestCase):
             self.assertFalse(t.ident is None)
             self.assertTrue(re.match('<TestThread\(.*, \w+ -?\d+\)>', repr(t)))
         if verbose:
-            print 'all tasks done'
+            print('all tasks done')
         self.assertEqual(numrunning.get(), 0)
 
     def test_ident_of_no_threading_threads(self):
@@ -116,7 +102,7 @@ class ThreadTests(BaseTestCase):
             done.set()
         done = threading.Event()
         ident = []
-        thread.start_new_thread(f, ())
+        _thread.start_new_thread(f, ())
         done.wait()
         self.assertFalse(ident[0] is None)
         # Kill the "immortal" _DummyThread
@@ -125,22 +111,26 @@ class ThreadTests(BaseTestCase):
     # run with a small(ish) thread stack size (256kB)
     def test_various_ops_small_stack(self):
         if verbose:
-            print 'with 256kB thread stack size...'
+            print('with 256kB thread stack size...')
         try:
             threading.stack_size(262144)
-        except thread.error:
-            self.skipTest('platform does not support changing thread stack size')
+        except _thread.error:
+            if verbose:
+                print('platform does not support changing thread stack size')
+            return
         self.test_various_ops()
         threading.stack_size(0)
 
     # run with a large thread stack size (1MB)
     def test_various_ops_large_stack(self):
         if verbose:
-            print 'with 1MB thread stack size...'
+            print('with 1MB thread stack size...')
         try:
             threading.stack_size(0x100000)
-        except thread.error:
-            self.skipTest('platform does not support changing thread stack size')
+        except _thread.error:
+            if verbose:
+                print('platform does not support changing thread stack size')
+            return
         self.test_various_ops()
         threading.stack_size(0)
 
@@ -154,11 +144,12 @@ class ThreadTests(BaseTestCase):
 
         mutex = threading.Lock()
         mutex.acquire()
-        tid = thread.start_new_thread(f, (mutex,))
+        tid = _thread.start_new_thread(f, (mutex,))
         # Wait for the thread to finish.
         mutex.acquire()
-        self.assertIn(tid, threading._active)
-        self.assertIsInstance(threading._active[tid], threading._DummyThread)
+        self.assertTrue(tid in threading._active)
+        self.assertTrue(isinstance(threading._active[tid],
+                                threading._DummyThread))
         del threading._active[tid]
 
     # PyThreadState_SetAsyncExc() is a CPython-only gimmick, not (currently)
@@ -167,7 +158,9 @@ class ThreadTests(BaseTestCase):
         try:
             import ctypes
         except ImportError:
-            self.skipTest('requires ctypes')
+            if verbose:
+                print("test_PyThreadState_SetAsyncExc can't import ctypes")
+            return  # can't do anything
 
         set_async_exc = ctypes.pythonapi.PyThreadState_SetAsyncExc
 
@@ -176,27 +169,6 @@ class ThreadTests(BaseTestCase):
 
         exception = ctypes.py_object(AsyncExc)
 
-        # First check it works when setting the exception from the same thread.
-        tid = thread.get_ident()
-
-        try:
-            result = set_async_exc(ctypes.c_long(tid), exception)
-            # The exception is async, so we might have to keep the VM busy until
-            # it notices.
-            while True:
-                pass
-        except AsyncExc:
-            pass
-        else:
-            # This code is unreachable but it reflects the intent. If we wanted
-            # to be smarter the above loop wouldn't be infinite.
-            self.fail("AsyncExc not raised")
-        try:
-            self.assertEqual(result, 1) # one thread state modified
-        except UnboundLocalError:
-            # The exception was raised too quickly for us to get the result.
-            pass
-
         # `worker_started` is set by the thread when it's inside a try/except
         # block waiting to catch the asynchronously set AsyncExc exception.
         # `worker_saw_exception` is set by the thread upon catching that
@@ -206,7 +178,7 @@ class ThreadTests(BaseTestCase):
 
         class Worker(threading.Thread):
             def run(self):
-                self.id = thread.get_ident()
+                self.id = _thread.get_ident()
                 self.finished = False
 
                 try:
@@ -221,32 +193,32 @@ class ThreadTests(BaseTestCase):
         t.daemon = True # so if this fails, we don't hang Python at shutdown
         t.start()
         if verbose:
-            print "    started worker thread"
+            print("    started worker thread")
 
         # Try a thread id that doesn't make sense.
         if verbose:
-            print "    trying nonsensical thread id"
+            print("    trying nonsensical thread id")
         result = set_async_exc(ctypes.c_long(-1), exception)
         self.assertEqual(result, 0)  # no thread states modified
 
         # Now raise an exception in the worker thread.
         if verbose:
-            print "    waiting for worker thread to get started"
+            print("    waiting for worker thread to get started")
         ret = worker_started.wait()
         self.assertTrue(ret)
         if verbose:
-            print "    verifying worker hasn't exited"
+            print("    verifying worker hasn't exited")
         self.assertTrue(not t.finished)
         if verbose:
-            print "    attempting to raise asynch exception in worker"
+            print("    attempting to raise asynch exception in worker")
         result = set_async_exc(ctypes.c_long(t.id), exception)
         self.assertEqual(result, 1) # one thread state modified
         if verbose:
-            print "    waiting for worker to say it caught the exception"
+            print("    waiting for worker to say it caught the exception")
         worker_saw_exception.wait(timeout=10)
         self.assertTrue(t.finished)
         if verbose:
-            print "    all OK -- joining worker"
+            print("    all OK -- joining worker")
         if t.finished:
             t.join()
         # else the thread is still running, and we have no way to kill it
@@ -254,12 +226,12 @@ class ThreadTests(BaseTestCase):
     def test_limbo_cleanup(self):
         # Issue 7481: Failure to start thread should cleanup the limbo map.
         def fail_new_thread(*args):
-            raise thread.error()
+            raise threading.ThreadError()
         _start_new_thread = threading._start_new_thread
         threading._start_new_thread = fail_new_thread
         try:
             t = threading.Thread(target=lambda: None)
-            self.assertRaises(thread.error, t.start)
+            self.assertRaises(threading.ThreadError, t.start)
             self.assertFalse(
                 t in threading._limbo,
                 "Failed to cleanup _limbo map on failure of Thread.start().")
@@ -273,13 +245,15 @@ class ThreadTests(BaseTestCase):
         try:
             import ctypes
         except ImportError:
-            self.skipTest('requires ctypes')
+            if verbose:
+                print("test_finalize_with_runnning_thread can't import ctypes")
+            return  # can't do anything
 
         rc = subprocess.call([sys.executable, "-c", """if 1:
-            import ctypes, sys, time, thread
+            import ctypes, sys, time, _thread
 
             # This lock is used as a simple event variable.
-            ready = thread.allocate_lock()
+            ready = _thread.allocate_lock()
             ready.acquire()
 
             # Module globals are cleared before __del__ is run
@@ -296,7 +270,7 @@ class ThreadTests(BaseTestCase):
                 ready.release()
                 time.sleep(100)
 
-            thread.start_new_thread(waitingThread, ())
+            _thread.start_new_thread(waitingThread, ())
             ready.acquire()  # Be sure the other thread is waiting.
             sys.exit(42)
             """])
@@ -313,7 +287,7 @@ class ThreadTests(BaseTestCase):
             def killer():
                 import os, time
                 time.sleep(2)
-                print 'program blocked; aborting'
+                print('program blocked; aborting')
                 os._exit(2)
             t = threading.Thread(target=killer)
             t.daemon = True
@@ -328,13 +302,11 @@ class ThreadTests(BaseTestCase):
             """],
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE)
-        self.addCleanup(p.stdout.close)
-        self.addCleanup(p.stderr.close)
         stdout, stderr = p.communicate()
         rc = p.returncode
         self.assertFalse(rc == 2, "interpreted was blocked")
         self.assertTrue(rc == 0,
-                        "Unexpected error: " + repr(stderr))
+                        "Unexpected error: " + ascii(stderr))
 
     def test_join_nondaemon_on_shutdown(self):
         # Issue 1722344
@@ -347,20 +319,18 @@ class ThreadTests(BaseTestCase):
                     sleep(1)
                     # As a non-daemon thread we SHOULD wake up and nothing
                     # should be torn down yet
-                    print "Woke up, sleep function is:", sleep
+                    print("Woke up, sleep function is:", sleep)
 
                 threading.Thread(target=child).start()
                 raise SystemExit
             """],
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE)
-        self.addCleanup(p.stdout.close)
-        self.addCleanup(p.stderr.close)
         stdout, stderr = p.communicate()
         self.assertEqual(stdout.strip(),
-            "Woke up, sleep function is: <built-in function sleep>")
-        stderr = re.sub(r"^\[\d+ refs\]", "", stderr, re.MULTILINE).strip()
-        self.assertEqual(stderr, "")
+            b"Woke up, sleep function is: <built-in function sleep>")
+        stderr = re.sub(br"^\[\d+ refs\]", b"", stderr, re.MULTILINE).strip()
+        self.assertEqual(stderr, b"")
 
     def test_enumerate_after_join(self):
         # Try hard to trigger #1703448: a thread is still returned in
@@ -368,7 +338,7 @@ class ThreadTests(BaseTestCase):
         enum = threading.enumerate
         old_interval = sys.getcheckinterval()
         try:
-            for i in xrange(1, 100):
+            for i in range(1, 100):
                 # Try a couple times at each thread-switching interval
                 # to get more interleavings.
                 sys.setcheckinterval(i // 5)
@@ -376,7 +346,7 @@ class ThreadTests(BaseTestCase):
                 t.start()
                 t.join()
                 l = enum()
-                self.assertNotIn(t, l,
+                self.assertFalse(t in l,
                     "#1703448 triggered after %d trials: %s" % (i, l))
         finally:
             sys.setcheckinterval(old_interval)
@@ -412,82 +382,21 @@ class ThreadTests(BaseTestCase):
                          msg=('%d references still around' %
                               sys.getrefcount(weak_raising_cyclic_object())))
 
-    @unittest.skipUnless(hasattr(os, 'fork'), 'test needs fork()')
-    def test_dummy_thread_after_fork(self):
-        # Issue #14308: a dummy thread in the active list doesn't mess up
-        # the after-fork mechanism.
-        code = """if 1:
-            import thread, threading, os, time
+    def test_old_threading_api(self):
+        # Just a quick sanity check to make sure the old method names are
+        # still present
+        t = threading.Thread()
+        t.isDaemon()
+        t.setDaemon(True)
+        t.getName()
+        t.setName("name")
+        t.isAlive()
+        e = threading.Event()
+        e.isSet()
+        threading.activeCount()
 
-            def background_thread(evt):
-                # Creates and registers the _DummyThread instance
-                threading.current_thread()
-                evt.set()
-                time.sleep(10)
 
-            evt = threading.Event()
-            thread.start_new_thread(background_thread, (evt,))
-            evt.wait()
-            assert threading.active_count() == 2, threading.active_count()
-            if os.fork() == 0:
-                assert threading.active_count() == 1, threading.active_count()
-                os._exit(0)
-            else:
-                os.wait()
-        """
-        _, out, err = assert_python_ok("-c", code)
-        self.assertEqual(out, '')
-        self.assertEqual(err, '')
-
-    @unittest.skipUnless(hasattr(os, 'fork'), "needs os.fork()")
-    def test_is_alive_after_fork(self):
-        # Try hard to trigger #18418: is_alive() could sometimes be True on
-        # threads that vanished after a fork.
-        old_interval = sys.getcheckinterval()
-
-        # Make the bug more likely to manifest.
-        sys.setcheckinterval(10)
-
-        try:
-            for i in range(20):
-                t = threading.Thread(target=lambda: None)
-                t.start()
-                pid = os.fork()
-                if pid == 0:
-                    os._exit(1 if t.is_alive() else 0)
-                else:
-                    t.join()
-                    pid, status = os.waitpid(pid, 0)
-                    self.assertEqual(0, status)
-        finally:
-            sys.setcheckinterval(old_interval)
-
-    def test_BoundedSemaphore_limit(self):
-        # BoundedSemaphore should raise ValueError if released too often.
-        for limit in range(1, 10):
-            bs = threading.BoundedSemaphore(limit)
-            threads = [threading.Thread(target=bs.acquire)
-                       for _ in range(limit)]
-            for t in threads:
-                t.start()
-            for t in threads:
-                t.join()
-            threads = [threading.Thread(target=bs.release)
-                       for _ in range(limit)]
-            for t in threads:
-                t.start()
-            for t in threads:
-                t.join()
-            self.assertRaises(ValueError, bs.release)
-
-class ThreadJoinOnShutdown(BaseTestCase):
-
-    # Between fork() and exec(), only async-safe functions are allowed (issues
-    # #12316 and #11870), and fork() from a worker thread is known to trigger
-    # problems with some operating systems (issue #3863): skip problematic tests
-    # on platforms known to behave badly.
-    platforms_to_skip = ('freebsd4', 'freebsd5', 'freebsd6', 'netbsd5',
-                         'os2emx')
+class ThreadJoinOnShutdown(unittest.TestCase):
 
     def _run_and_join(self, script):
         script = """if 1:
@@ -496,12 +405,15 @@ class ThreadJoinOnShutdown(BaseTestCase):
             # a thread, which waits for the main program to terminate
             def joiningfunc(mainthread):
                 mainthread.join()
-                print 'end of thread'
+                print('end of thread')
+                # stdout is fully buffered because not a tty, we have to flush
+                # before exit.
+                sys.stdout.flush()
         \n""" + script
 
         p = subprocess.Popen([sys.executable, "-c", script], stdout=subprocess.PIPE)
         rc = p.wait()
-        data = p.stdout.read().replace('\r', '')
+        data = p.stdout.read().decode().replace('\r', '')
         p.stdout.close()
         self.assertEqual(data, "end of main\nend of thread\n")
         self.assertFalse(rc == 2, "interpreter was blocked")
@@ -515,15 +427,16 @@ class ThreadJoinOnShutdown(BaseTestCase):
                                  args=(threading.current_thread(),))
             t.start()
             time.sleep(0.1)
-            print 'end of main'
+            print('end of main')
             """
         self._run_and_join(script)
 
 
-    @unittest.skipUnless(hasattr(os, 'fork'), "needs os.fork()")
-    @unittest.skipIf(sys.platform in platforms_to_skip, "due to known OS bug")
     def test_2_join_in_forked_process(self):
         # Like the test above, but from a forked interpreter
+        import os
+        if not hasattr(os, 'fork'):
+            return
         script = """if 1:
             childpid = os.fork()
             if childpid != 0:
@@ -533,15 +446,23 @@ class ThreadJoinOnShutdown(BaseTestCase):
             t = threading.Thread(target=joiningfunc,
                                  args=(threading.current_thread(),))
             t.start()
-            print 'end of main'
+            print('end of main')
             """
         self._run_and_join(script)
 
-    @unittest.skipUnless(hasattr(os, 'fork'), "needs os.fork()")
-    @unittest.skipIf(sys.platform in platforms_to_skip, "due to known OS bug")
     def test_3_join_in_forked_from_thread(self):
         # Like the test above, but fork() was called from a worker thread
         # In the forked process, the main Thread object must be marked as stopped.
+        import os
+        if not hasattr(os, 'fork'):
+            return
+        # Skip platforms with known problems forking from a worker thread.
+        # See http://bugs.python.org/issue3863.
+        if sys.platform in ('freebsd4', 'freebsd5', 'freebsd6', 'netbsd5',
+                           'os2emx'):
+            print('Skipping test_3_join_in_forked_from_thread'
+                  ' due to known OS bugs on', sys.platform, file=sys.stderr)
+            return
         script = """if 1:
             main_thread = threading.current_thread()
             def worker():
@@ -552,7 +473,7 @@ class ThreadJoinOnShutdown(BaseTestCase):
 
                 t = threading.Thread(target=joiningfunc,
                                      args=(main_thread,))
-                print 'end of main'
+                print('end of main')
                 t.start()
                 t.join() # Should not block: main_thread is already stopped
 
@@ -570,11 +491,15 @@ class ThreadJoinOnShutdown(BaseTestCase):
         self.assertEqual(data, expected_output)
 
     @unittest.skipUnless(hasattr(os, 'fork'), "needs os.fork()")
-    @unittest.skipIf(sys.platform in platforms_to_skip, "due to known OS bug")
     def test_4_joining_across_fork_in_worker_thread(self):
         # There used to be a possible deadlock when forking from a child
         # thread.  See http://bugs.python.org/issue6643.
 
+        # Skip platforms with known problems forking from a worker thread.
+        # See http://bugs.python.org/issue3863.
+        if sys.platform in ('freebsd4', 'freebsd5', 'freebsd6', 'os2emx'):
+            raise unittest.SkipTest('due to known OS bugs on ' + sys.platform)
+
         # The script takes the following steps:
         # - The main thread in the parent process starts a new thread and then
         #   tries to join it.
@@ -643,7 +568,6 @@ class ThreadJoinOnShutdown(BaseTestCase):
         self.assertScriptHasOutput(script, "end of main\n")
 
     @unittest.skipUnless(hasattr(os, 'fork'), "needs os.fork()")
-    @unittest.skipIf(sys.platform in platforms_to_skip, "due to known OS bug")
     def test_5_clear_waiter_locks_to_avoid_crash(self):
         # Check that a spawned thread that forks doesn't segfault on certain
         # platforms, namely OS X.  This used to happen if there was a waiter
@@ -656,6 +580,10 @@ class ThreadJoinOnShutdown(BaseTestCase):
         # lock will be acquired, we can't know if the internal mutex will be
         # acquired at the time of the fork.
 
+        # Skip platforms with known problems forking from a worker thread.
+        # See http://bugs.python.org/issue3863.
+        if sys.platform in ('freebsd4', 'freebsd5', 'freebsd6', 'os2emx'):
+            raise unittest.SkipTest('due to known OS bugs on ' + sys.platform)
         script = """if True:
             import os, time, threading
 
@@ -700,72 +628,8 @@ class ThreadJoinOnShutdown(BaseTestCase):
         output = "end of worker thread\nend of main thread\n"
         self.assertScriptHasOutput(script, output)
 
-    @unittest.skipUnless(hasattr(os, 'fork'), "needs os.fork()")
-    @unittest.skipIf(sys.platform in platforms_to_skip, "due to known OS bug")
-    def test_reinit_tls_after_fork(self):
-        # Issue #13817: fork() would deadlock in a multithreaded program with
-        # the ad-hoc TLS implementation.
 
-        def do_fork_and_wait():
-            # just fork a child process and wait it
-            pid = os.fork()
-            if pid > 0:
-                os.waitpid(pid, 0)
-            else:
-                os._exit(0)
-
-        # start a bunch of threads that will fork() child processes
-        threads = []
-        for i in range(16):
-            t = threading.Thread(target=do_fork_and_wait)
-            threads.append(t)
-            t.start()
-
-        for t in threads:
-            t.join()
-
-    @cpython_only
-    @unittest.skipIf(_testcapi is None, "need _testcapi module")
-    def test_frame_tstate_tracing(self):
-        # Issue #14432: Crash when a generator is created in a C thread that is
-        # destroyed while the generator is still used. The issue was that a
-        # generator contains a frame, and the frame kept a reference to the
-        # Python state of the destroyed C thread. The crash occurs when a trace
-        # function is setup.
-
-        def noop_trace(frame, event, arg):
-            # no operation
-            return noop_trace
-
-        def generator():
-            while 1:
-                yield "genereator"
-
-        def callback():
-            if callback.gen is None:
-                callback.gen = generator()
-            return next(callback.gen)
-        callback.gen = None
-
-        old_trace = sys.gettrace()
-        sys.settrace(noop_trace)
-        try:
-            # Install a trace function
-            threading.settrace(noop_trace)
-
-            # Create a generator in a C thread which exits after the call
-            _testcapi.call_in_temporary_c_thread(callback)
-
-            # Call the generator in a different Python thread, check that the
-            # generator didn't keep a reference to the destroyed thread state
-            for test in range(3):
-                # The trace function is still called here
-                callback()
-        finally:
-            sys.settrace(old_trace)
-
-
-class ThreadingExceptionTests(BaseTestCase):
+class ThreadingExceptionTests(unittest.TestCase):
     # A RuntimeError should be raised if Thread.start() is called
     # multiple times.
     def test_start_thread_again(self):
@@ -842,13 +706,13 @@ class BoundedSemaphoreTests(lock_tests.BoundedSemaphoreTests):
         self.assertEqual(data, expected_output)
 
 def test_main():
-    test.test_support.run_unittest(LockTests, RLockTests, EventTests,
-                                   ConditionAsRLockTests, ConditionTests,
-                                   SemaphoreTests, BoundedSemaphoreTests,
-                                   ThreadTests,
-                                   ThreadJoinOnShutdown,
-                                   ThreadingExceptionTests,
-                                   )
+    test.support.run_unittest(LockTests, RLockTests, EventTests,
+                              ConditionAsRLockTests, ConditionTests,
+                              SemaphoreTests, BoundedSemaphoreTests,
+                              ThreadTests,
+                              ThreadJoinOnShutdown,
+                              ThreadingExceptionTests,
+                              )
 
 if __name__ == "__main__":
     test_main()
